/* LED counter GNOME panel applet
 * Insprired by Sixtyfour bit counter by Tim Gerla
 * and xled by Jan Schoenepauck <schoenep@uni-wuppertal.de>
 * and Joachim Gassen <gassen@uni-muenster.de>"
 *
 * By Uwe Steinmann <Uwe.Steinmann@fernuni-hagen.de>
 *
 * Enjoy.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License  
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gnome.h>
#include <applet-widget.h>
#include <math.h>
#include <glib.h>
#include "ledcount.h"

static void
my_list_print(gpointer list_data, gpointer user_data) {
	DISP_INFO *disp;

	disp = (DISP_INFO *) list_data;
	g_print("%d - %s\n", disp->number, disp->title);
}

DISP_INFO *
add_disp(GList **displist, DISP_INFO *disp_info) {
	DISP_INFO *disp;
	GList *list;
	if(NULL == (disp = g_malloc(sizeof(DISP_INFO))))
		return NULL;
	disp->display = NULL;
	disp->container = NULL;
	disp->label = NULL;
	disp->button = NULL;
	if(disp_info) {
		if(disp_info->title) disp->title = g_strdup(disp_info->title);
		if(disp_info->mailbox_file) disp->mailbox_file = g_strdup(disp_info->mailbox_file);
		if(disp_info->backend_command) disp->backend_command = g_strdup(disp_info->backend_command);
		if(disp_info->exec_command) disp->exec_command = g_strdup(disp_info->exec_command);
		if(disp_info->imap_server) disp->imap_server = g_strdup(disp_info->imap_server);
		if(disp_info->imap_username) disp->imap_username = g_strdup(disp_info->imap_username);
		if(disp_info->imap_password) disp->imap_password = g_strdup(disp_info->imap_password);
		disp->updatetime = disp_info->updatetime;
		disp->preset = disp_info->preset;
		disp->count_mode = disp_info->count_mode;
		disp->digits = disp_info->digits;
		disp->exec_button = disp_info->exec_button;
		disp->show_all_messages = disp_info->show_all_messages;
		disp->timeout = disp_info->timeout;
		disp->autodigits = disp_info->autodigits;
	} else {
		disp->title = g_strdup("Title");
		disp->mailbox_file = g_strdup("");
		disp->backend_command = g_strdup("");
		disp->exec_command = g_strdup("");
		disp->imap_server = g_strdup("");
		disp->imap_username = g_strdup("");
		disp->imap_password = g_strdup("");
		disp->updatetime = 10000;
		disp->preset = 0;
		disp->count_mode = 1;
		disp->digits = 2;
		disp->exec_button = 1;
		disp->show_all_messages = 0;
		disp->timeout = 0;
		disp->autodigits = 1;
	}
	list = g_list_append (g_list_last(*displist), disp);
	if(*displist == NULL)
		*displist = list;
//	g_print("--------------------------\nNumber of elements: %d\n",
//	                              g_list_length(g_list_first(*displist)));
//	g_list_foreach(g_list_first(*displist), my_list_print, NULL);
	return(disp);
}

DISP_INFO *
get_disp(GList *displist, int number)
{
	GList *list;

	if(displist == NULL)
		return NULL;
	list = g_list_first(displist);
	while(list) {
		if(((DISP_INFO *) list->data)->number == number)
			return((DISP_INFO *) list->data);
		list = list->next;
	}
	return(NULL);
}

DISP_INFO *
get_last_disp(GList *displist)
{
	GList *list;

	if(displist == NULL)
		return NULL;
	list = g_list_last(displist);
	return((DISP_INFO *) list->data);
}

DISP_INFO *
get_first_disp(GList *displist)
{
	GList *list;

	if(displist == NULL)
		return NULL;
	list = g_list_first(displist);
	return((DISP_INFO *) list->data);
}

DISP_INFO *
get_next_disp(GList *displist, DISP_INFO *disp_info)
{
	GList *list;

	if(displist == NULL)
		return NULL;
	list = g_list_find(g_list_first(displist), disp_info);
	if(list) {
		list = g_list_next(list);
		if(list)
			return((DISP_INFO *) list->data);
		else	
			return NULL;
	} else
		return NULL;
}

DISP_INFO *
get_prev_disp(GList *displist, DISP_INFO *disp_info)
{
	GList *list;

	if(displist == NULL)
		return NULL;
	list = g_list_find(g_list_first(displist), disp_info);
	if(list) {
		list = g_list_previous(list);
		if(list)
			return((DISP_INFO *) list->data);
		else	
			return NULL;
	} else
		return NULL;
}

GList *
remove_disp(GList *displist, DISP_INFO *disp_info)
{
	GList *list;

	if(displist == NULL)
		return NULL;
	list = g_list_first(displist);
	while(list) {
		if(((DISP_INFO *) list->data) == disp_info) {
			if(disp_info->title) g_free(disp_info->title);
			if(disp_info->mailbox_file) g_free(disp_info->mailbox_file);
			if(disp_info->exec_command) g_free(disp_info->exec_command);
			if(disp_info->backend_command) g_free(disp_info->backend_command);
			if(disp_info->imap_server) g_free(disp_info->imap_server);
			if(disp_info->imap_username) g_free(disp_info->imap_username);
			if(disp_info->imap_password) g_free(disp_info->imap_password);
			displist = g_list_remove(g_list_first(displist), (gpointer) disp_info);
			g_free(disp_info);
//	g_print("--------------------------\nNumber of elements: %d\n", g_list_length(g_list_first(displist)));
//	g_list_foreach(g_list_first(displist), my_list_print, NULL);
			return displist;
		}
		list = list->next;
	}
	return displist;
}

guint
get_number_of_disp(GList *displist) {
	return(g_list_length(g_list_first(displist)));
}

#define SWAP_FIELD(field,i) i = disp_info1->field; \
	disp_info1->field = disp_info2->field; \
	disp_info2->field = i;

void
switch_disp_data(DISP_INFO *disp_info1, DISP_INFO *disp_info2) {
	int i;
	char *str;
	time_t time;

	SWAP_FIELD(digits,i)
	SWAP_FIELD(autodigits,i)
	SWAP_FIELD(leading_zeros,i)
	SWAP_FIELD(disptype,i)
	SWAP_FIELD(mailbox_file,str)
	SWAP_FIELD(imap_server,str)
	SWAP_FIELD(imap_username,str)
	SWAP_FIELD(imap_password,str)
	SWAP_FIELD(show_all_messages,i)
	SWAP_FIELD(count_mode,i)
	SWAP_FIELD(front_end_mode,i)
	SWAP_FIELD(backend_command,str)
	SWAP_FIELD(exec_command,str)
	SWAP_FIELD(exec_button,i)
	SWAP_FIELD(title,str)
	SWAP_FIELD(timeout,i)
	SWAP_FIELD(updatetime,i)
	SWAP_FIELD(preset,i)
	SWAP_FIELD(lastcount,i)
	SWAP_FIELD(lasttime,time)
	SWAP_FIELD(indicate_counting,i)
}
