/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * GtkDisplay: Emulate a simple 7 segment, multi digit display
 * Copyright (C) 1999 Uwe Steinmann
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef __GTK_DISPLAY_H__
#define __GTK_DISPLAY_H__


#include <gdk/gdk.h>
#include <gtk/gtkmisc.h>
#include "gtkled.h"


#ifdef __cplusplus
extern "C" {
#pragma }
#endif /* __cplusplus */


typedef enum {
	DispAlphaNumeric,
	DispSevenSegment
} DISPTYPE;

#define GTK_DISPLAY(obj)	      (GTK_CHECK_CAST (obj, gtk_display_get_type (), GtkDisplay))
#define GTK_DISPLAY_CLASS(klass)  (GTK_CHECK_CLASS_CAST (klass, gtk_display_get_type (), GtkDisplayClass))
#define GTK_IS_DISPLAY(obj)	      (GTK_CHECK_TYPE (obj, gtk_display_get_type ()))

typedef struct _GtkDisplay	GtkDisplay;
typedef struct _GtkDisplayClass	GtkDisplayClass;

#define DIGITS 3 
#define MAX_DIGITS 100

struct _GtkDisplay
{
  GtkVBox  misc;
  GtkWidget *led[MAX_DIGITS];
  GtkWidget *table;
  GdkColor fg[3];
  GdkColor bg[3];

	gint valtype;
  gint value;
	char *strvalue;
  guint digits;
  guint autodigits;
  guint leadingzeros;
  DISPTYPE displaytype;
};

struct _GtkDisplayClass
{
  GtkVBoxClass parent_class;

	void (* pressed) (GtkDisplay *display);
};


GtkType	   gtk_display_get_type    (void);
GtkWidget* gtk_display_new	       (void);
GtkWidget* gtk_display_new_with_digits	       (guint digits);
void	   gtk_display_set_state   (GtkDisplay			*display,
				GtkStateType		widget_state,
				guint			value);
void	   gtk_display_set_digits   (GtkDisplay			*display,
				guint			value);
gint	   gtk_display_get_digits (GtkDisplay *display);
void	   gtk_display_set_value   (GtkDisplay			*display,
				gint			value);
gint      gtk_display_get_value   (GtkDisplay			*display);
void	   gtk_display_set_strvalue   (GtkDisplay			*display,
				char			*strvalue);
char*      gtk_display_get_strvalue   (GtkDisplay			*display);
void       gtk_display_set_colors  (GtkDisplay                  *display,
				GdkColor                *active,
				GdkColor                *inactive,
				GdkColor				*over);
void gtk_display_set_autorange (GtkDisplay *display, guint value);
gint gtk_display_get_autorange (GtkDisplay *display);
void gtk_display_set_leadingzeros (GtkDisplay *display, guint value);
gint gtk_display_get_leadingzeros (GtkDisplay *display);
void gtk_display_set_displaytype (GtkDisplay *display, DISPTYPE value);
DISPTYPE gtk_display_get_displaytype (GtkDisplay *display);
void gtk_display_pressed (GtkDisplay *display);

#ifdef __cplusplus
#pragma {
}
#endif /* __cplusplus */


#endif /* __GTK_DISPLAY_H__ */
