/* LED counter GNOME panel applet
 * Insprired by Sixtyfour bit counter by Tim Gerla
 * and xled by Jan Schoenepauck <schoenep@uni-wuppertal.de>
 * and Joachim Gassen <gassen@uni-muenster.de>"
 *
 * By Uwe Steinmann <Uwe.Steinmann@fernuni-hagen.de>
 *
 * Enjoy.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or 
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License  
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/* $Id: ledcount.c,v 1.19 08/22/00 19:35:30 MEST steinm Exp $ */

#include <gnome.h>
#include <applet-widget.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "gtkdisplay.h"
#include "ledcount.h"
#include "disp_manage.h"
#include "imap.h"


int check_vbox_messages (void);
int count_mail (char *file, int all, time_t lasttime);
int count_timer(GtkWidget *data);
void save_config(gchar *privcfgpath, CountData *cd);

extern void prefs_cb(GtkWidget *w, gpointer p);

static void
about_cb(GtkWidget *w, gpointer d)
{
	gchar *authors[5];
	GtkWidget *about;
	
	authors[0] = "Uwe Steinmann";
	authors[1] = _("Insprired by Sixtyfour bit counter by Tim Gerla");
	authors[2] = _("and xled by Jan Schoenepauck <schoenep@uni-wuppertal.de>");
	authors[3] = _("and Joachim Gassen <gassen@uni-muenster.de>");
	authors[4] = NULL;
	
	about = gnome_about_new (
		"LED Counter",
		VERSION,
		"(C) 1999 Uwe Steinmann",
		(const gchar**)authors,
		_("This is a general purpose counter. It displays the counted number"
		" on an old fashion 7 segment display. It can count mail messages"
		" or simple display the value written to stdout by any command, making"
	  " it possible to count almost everything."
		" The number of displays is unlimited but each may not have more than"
    " 10 digits."),
		NULL);
	
	gtk_widget_show(about);
	
	return;
}

void
change_spacing(CountData *cd, int space)
{
	switch (cd->orient) {
		case GNOME_Panel_ORIENT_LEFT:
		case GNOME_Panel_ORIENT_RIGHT:
//			gtk_table_set_row_spacings(GTK_TABLE(counter), space);
			gtk_box_set_spacing(GTK_BOX(cd->counter), space);
			break;
		default:
//			gtk_table_set_col_spacings(GTK_TABLE(counter), space);
			gtk_box_set_spacing(GTK_BOX(cd->counter), space);
			break;
	}
}

int
count_timer(GtkWidget *data)
{
	gint count = 0;
	char *str = NULL;
	DISP_INFO *disp_info = (DISP_INFO *) data;
	CountData *cd = (CountData *) disp_info->parent;

	if(disp_info->indicate_counting)
		gtk_display_set_colors(GTK_DISPLAY(disp_info->display), &(cd->over), &(cd->off), &(cd->on));
	/**** If data = NULL count for all displays *****/
//	disp_info = get_first_disp(cd->displist);
// 	while(disp_info) {
//		if((cd->countthis == -1) || (disp_info->number == cd->countthis)) {
			switch(disp_info->count_mode) {
				case MODE_MAILBOX: { /* mailbox */
					struct stat statbuf;
					count = disp_info->lastcount;
					if(stat(disp_info->mailbox_file, &statbuf)) {
						count = 996;
						break;
					}
//fprintf(stderr, "mailbox = %s, mtime = %d, lasttime = %d\n", disp_info->mailbox_file, statbuf.st_ctime, disp_info->lasttime);
					if(statbuf.st_ctime == disp_info->lasttime)
						break;
					count = count_mail(disp_info->mailbox_file,
					                   disp_info->show_all_messages,
					                   disp_info->lasttime);
					disp_info->lasttime = statbuf.st_ctime;
					disp_info->lastcount = count;
					break;
					}
				case MODE_COMMAND: {/* command */
					char *buf;
					int buflen = 0;
					int l=0;

					FILE *fp = popen(disp_info->backend_command, "r");
					if(NULL == fp) {
						count = 0;
						break;
					}
//						fscanf(fp, "%d", &count);
#define EXEC_INPUT_BUF 40
					if(NULL == (buf = (char *) g_malloc(EXEC_INPUT_BUF))) {
						count = 0;
						break;
					}
					str = buf;
					buflen = EXEC_INPUT_BUF;
					buf[0] = '\0';
					while ( !feof(fp) || l != 0 ) {
						l = 0;
						do {
							if ( buflen <= (l+1) ) {
								buf = realloc(buf, buflen + EXEC_INPUT_BUF);
								if ( buf == NULL ) {
									count = 0;
									break;
								}
								buflen += EXEC_INPUT_BUF;
							}
							if ( fgets(&(buf[l]), buflen - l, fp) == NULL ) {
								/* eof */
								break;
							}
							l += strlen(&(buf[l]));
						} while ( (l > 0) && (buf[l-1] != '\n') );
						if ( feof(fp) && (l == 0) ) {
							break;
						}
					}
					pclose(fp);
#undef EXEC_INPUT_BUF
					break;
				}
				case MODE_IMAP: { /* Imap */
					ImapContext *imap;
					imap = imap_login(disp_info->imap_server,
					                  disp_info->imap_username,
					                  disp_info->imap_password);
					if(NULL == imap)
						count = -1;
					else if(imap->error)
						count = -2;
					else if(strcmp(disp_info->mailbox_file, "")) 
						count = imap_get_recent(imap, disp_info->mailbox_file);
					else
						count = imap_get_recent(imap, "INBOX");
      		imap_destroy(imap);
					break;
				}
				case MODE_VBOX: /* vbox messages */
					count = check_vbox_messages();
					break;
			}

			if(disp_info->display) {
				if(str)
					gtk_display_set_strvalue(GTK_DISPLAY(disp_info->display), str);
				else
					gtk_display_set_value(GTK_DISPLAY(disp_info->display), count+disp_info->preset);
				if(gtk_display_get_autorange(GTK_DISPLAY(disp_info->display)))
					gtk_widget_set_usize((GTK_WIDGET(disp_info->display)),
			  		gtk_display_get_digits(GTK_DISPLAY(disp_info->display)) * cd->width,
			  		cd->height);
			}
//		}
//		disp_info = get_next_disp(cd->displist, disp_info);
//	}
	if(disp_info->indicate_counting)
		gtk_display_set_colors(GTK_DISPLAY(disp_info->display), &(cd->on), &(cd->off), &(cd->over));

	return 1;
}

gint run_command(GtkWidget *widget, gpointer data) {
  DISP_INFO   *disp_info;

	disp_info = (DISP_INFO *) data;
	if(disp_info->exec_command) {
		system(disp_info->exec_command);
  }

	return 0;
}

gint button_pressed_cb(GtkWidget *widget, GdkEventButton *event, gpointer data) {
  DISP_INFO   *disp_info;

	disp_info = (DISP_INFO *) data;
	return 0;
}

void remove_exec_button(DISP_INFO *disp_info)
{
	gtk_widget_destroy(disp_info->label);
}

void add_exec_button(DISP_INFO *disp_info)
{
	disp_info->label = gtk_label_new(disp_info->title);
	gtk_widget_set_usize(disp_info->label, 1, BUTTON_HEIGHT);
	gtk_box_pack_end(GTK_BOX(disp_info->container), disp_info->label, TRUE, TRUE, 1);
	gtk_widget_show_all(disp_info->container);
}

/*
void destroy_counter(GtkWidget *widget)
{
	if(widget == NULL)
		return;
	gtk_widget_destroy(widget);
	counter = NULL;
}
*/

void
add_new_display(CountData *cd, GtkWidget *table, DISP_INFO *disp_info) {
	disp_info->frame = gtk_frame_new(NULL);
	gtk_container_set_border_width(GTK_CONTAINER(disp_info->frame), 1);
	gtk_frame_set_shadow_type(GTK_FRAME(disp_info->frame), GTK_SHADOW_IN);

	disp_info->container = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(disp_info->frame), disp_info->container);

	disp_info->display = gtk_display_new_with_digits(disp_info->digits);
	gtk_display_set_autorange(GTK_DISPLAY(disp_info->display), disp_info->autodigits);
	gtk_display_set_leadingzeros(GTK_DISPLAY(disp_info->display), disp_info->leading_zeros);
	gtk_display_set_displaytype(GTK_DISPLAY(disp_info->display), disp_info->disptype);
	count_timer((GtkWidget *) disp_info);

	/**** Display ****/
	disp_info->button = gtk_button_new();
	GTK_WIDGET_UNSET_FLAGS(disp_info->button, GTK_CAN_DEFAULT);
	GTK_WIDGET_UNSET_FLAGS(disp_info->button, GTK_CAN_FOCUS);
	disp_info->tip = (GtkWidget *)gtk_tooltips_new();
	gtk_tooltips_set_tip(GTK_TOOLTIPS(disp_info->tip), disp_info->button, disp_info->title, NULL);
	gtk_tooltips_enable(GTK_TOOLTIPS(disp_info->tip));

	gtk_signal_connect(GTK_OBJECT(disp_info->button), "clicked",
         	  GTK_SIGNAL_FUNC(run_command),
         	  (void *) disp_info);
	gtk_container_set_border_width(GTK_CONTAINER(disp_info->button), 0);
	gtk_container_add(GTK_CONTAINER(disp_info->button), disp_info->display);
	gtk_display_set_colors(GTK_DISPLAY(disp_info->display), &(cd->on), &(cd->off), &(cd->over));
	gtk_widget_set_usize(disp_info->display,
	   cd->width*gtk_display_get_digits(GTK_DISPLAY(disp_info->display)),
	   cd->height);
	gtk_box_pack_start(GTK_BOX(disp_info->container), disp_info->button, TRUE, TRUE, 0);

	/**** Button ****/
	if(disp_info->exec_button) {
		add_exec_button(disp_info);
	} else {
		gtk_widget_set_usize(disp_info->display,
		   cd->width*gtk_display_get_digits(GTK_DISPLAY(disp_info->display)),
		   cd->height+BUTTON_HEIGHT);
	}
	gtk_box_pack_start(GTK_BOX(table), disp_info->frame, TRUE, TRUE, 0);
}

static GtkWidget
*create_counter(CountData *cd)
{
	GtkWidget *table;
	DISP_INFO *disp_info;
	
	gdk_color_parse(cd->coff_s, &(cd->off));
	gdk_color_parse(cd->con_s, &(cd->on));
	gdk_color_parse(cd->cover_s, &(cd->over));

	switch (cd->orient) {
		case GNOME_Panel_ORIENT_LEFT:
		case GNOME_Panel_ORIENT_RIGHT:
			table = gtk_vbox_new(FALSE, cd->spacing);
			break;
		default:
			table = gtk_hbox_new(FALSE, cd->spacing);
			break;
	}

	disp_info = get_first_disp(cd->displist);
 	while(disp_info) {
		add_new_display(cd, table, disp_info);
		disp_info = get_next_disp(cd->displist, disp_info);
	}

	gtk_widget_show_all(table);
	return table;
}

static void
applet_change_orient(GtkWidget *w, PanelOrientType o, gpointer data)
{
	DISP_INFO *disp_info;
	CountData *cd = data;

fprintf(stderr, "change orient %s\n", (o == ORIENT_LEFT || o == ORIENT_RIGHT) ? "left right" : "up down");

	gtk_container_remove(GTK_CONTAINER(cd->globalbox), cd->counter);

	switch (o) {
		case ORIENT_LEFT:
		case ORIENT_RIGHT:
			cd->counter = gtk_vbox_new(FALSE, 0);
			break;
		default:
			cd->counter = gtk_hbox_new(FALSE, 0);
			break;
	}

	disp_info = get_first_disp(cd->displist);
 	while(disp_info) {
		gtk_box_pack_start(GTK_BOX(cd->counter), disp_info->container, TRUE, TRUE, 0);
		disp_info = get_next_disp(cd->displist, disp_info);
	}
	gtk_container_add(GTK_CONTAINER(cd->globalbox), cd->counter);
}

static void
bye(GtkWidget *w, gpointer p)
{
//	CountData *cd = p;
//	save_config(APPLET_WIDGET(w)->privcfgpath, cd);
	applet_widget_gtk_main_quit();
	exit(0);
}

#define EXTRA_SECTION 
void
save_config(gchar *privcfgpath, CountData *cd) {
	int i;
	int displays;
	char buffer[300];
	DISP_INFO *disp_info;

	gnome_config_push_prefix (privcfgpath);
fprintf(stderr, "save_session: path = %s\n", privcfgpath);
	displays = get_number_of_disp(cd->displist);
	gnome_config_set_int("ledcount/"EXTRA_SECTION"displays", displays);
	gnome_config_set_int("ledcount/"EXTRA_SECTION"width", cd->width);
	gnome_config_set_int("ledcount/"EXTRA_SECTION"height", cd->height);
	gnome_config_set_int("ledcount/"EXTRA_SECTION"spacing", cd->spacing);
	sprintf(buffer, "ledcount/"EXTRA_SECTION"color_on");
	gnome_config_set_string(buffer, cd->con_s);
	sprintf(buffer, "ledcount/"EXTRA_SECTION"color_off");
	gnome_config_set_string(buffer, cd->coff_s);
	sprintf(buffer, "ledcount/"EXTRA_SECTION"color_over");
	gnome_config_set_string(buffer, cd->cover_s);
	i = 0;
	disp_info = get_first_disp(cd->displist);
 	while(disp_info) {
		sprintf(buffer, "ledcount/"EXTRA_SECTION"mailbox%d", i+1);
		gnome_config_set_string(buffer, disp_info->mailbox_file);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"execute%d", i+1);
		gnome_config_set_string(buffer, disp_info->exec_command);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"title%d", i+1);
		gnome_config_set_string(buffer, disp_info->title);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"backend%d", i+1);
		gnome_config_set_string(buffer, disp_info->backend_command);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"imapserver%d", i+1);
		gnome_config_set_string(buffer, disp_info->imap_server);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"imapusername%d", i+1);
		gnome_config_set_string(buffer, disp_info->imap_username);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"imappassword%d", i+1);
		gnome_config_set_string(buffer, disp_info->imap_password);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"digits%d", i+1);
		gnome_config_set_int(buffer, disp_info->digits);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"countmode%d", i+1);
		gnome_config_set_int(buffer, disp_info->count_mode);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"showall%d", i+1);
		gnome_config_set_int(buffer, disp_info->show_all_messages);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"frontendmode%d", i+1);
		gnome_config_set_int(buffer, disp_info->front_end_mode);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"autorange%d", i+1);
		gnome_config_set_int(buffer, disp_info->autodigits);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"button%d", i+1);
		gnome_config_set_int(buffer, disp_info->exec_button);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"updatetime%d", i+1);
		gnome_config_set_int(buffer, disp_info->updatetime);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"preset%d", i+1);
		gnome_config_set_int(buffer, disp_info->preset);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"indicate%d", i+1);
		gnome_config_set_int(buffer, disp_info->indicate_counting);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"leadingzeros%d", i+1);
		gnome_config_set_int(buffer, disp_info->leading_zeros);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"disptype%d", i+1);
		gnome_config_set_int(buffer, (int) disp_info->disptype);
		i++;
		disp_info = get_next_disp(cd->displist, disp_info);
	}
	gnome_config_pop_prefix ();
	gnome_config_sync();
	gnome_config_drop_all();
}

static int
save_session(GtkWidget *w, gchar *privcfgpath, gchar *globcfgpath, gpointer p)
{
	CountData *cd = (CountData *) p;
	save_config(privcfgpath, cd);
	return FALSE;
}

void
load_config(gchar *privcfgpath, CountData *cd) {
	int i;
	int displays;
	
	gnome_config_push_prefix (privcfgpath);
fprintf(stderr, "load: path = %s\n", privcfgpath);
	displays = gnome_config_get_int("ledcount/"EXTRA_SECTION"displays=1");
	if(displays <= 0)
		displays = 1;
	cd->width = gnome_config_get_int("ledcount/"EXTRA_SECTION"width=21");
	cd->height = gnome_config_get_int("ledcount/"EXTRA_SECTION"height=30");
	cd->spacing = gnome_config_get_int("ledcount/"EXTRA_SECTION"spacing=5");
	sprintf(cd->con_s, "%s", gnome_config_get_string("ledcount/"EXTRA_SECTION"color_on=#FF1710"));
	sprintf(cd->coff_s, "%s", gnome_config_get_string("ledcount/"EXTRA_SECTION"color_off=#400000"));
	sprintf(cd->cover_s, "%s", gnome_config_get_string("ledcount/"EXTRA_SECTION"color_over=#17FF00"));

	for(i=0; i<displays; i++) {
// FIXME: maybe we have to use strdup here because apply_cb frees the memory
		char buffer[70];
		DISP_INFO *disp_info;
		if(NULL == (disp_info = add_disp(&(cd->displist), NULL)))
			continue;
		cd->displays++;
		disp_info->parent = cd;
		disp_info->number = i+1;
		sprintf(buffer, "ledcount/"EXTRA_SECTION"mailbox%d=", i+1);
		disp_info->mailbox_file = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"execute%d=", i+1);
		disp_info->exec_command = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"title%d=%d", i+1, i+1);
		disp_info->title = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"backend%d=", i+1);
		disp_info->backend_command = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"imapserver%d=", i+1);
		disp_info->imap_server = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"imapusername%d=", i+1);
		disp_info->imap_username = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"imappassword%d=", i+1);
		disp_info->imap_password = gnome_config_get_string(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"digits%d=3", i+1);
		disp_info->digits = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"countmode%d=1", i+1);
		disp_info->count_mode = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"showall%d=0", i+1);
		disp_info->show_all_messages = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"frontendmode%d=0", i+1);
		disp_info->front_end_mode = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"autorange%d=0", i+1);
		disp_info->autodigits = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"button%d=1", i+1);
		disp_info->exec_button = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"updatetime%d=100000", i+1);
		disp_info->updatetime = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"preset%d=0", i+1);
		disp_info->preset = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"indicate%d=0", i+1);
		disp_info->indicate_counting = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"leadingzeros%d=0", i+1);
		disp_info->leading_zeros = gnome_config_get_int(buffer);
		sprintf(buffer, "ledcount/"EXTRA_SECTION"disptype%d=0", i+1);
		disp_info->disptype = gnome_config_get_int(buffer);
		disp_info->timeout = gtk_timeout_add(disp_info->updatetime, (GtkFunction)count_timer, disp_info);
		disp_info->lastcount = 0;
		disp_info->lasttime = 0;
	}

	gnome_config_pop_prefix ();
}

static CountData*
create_count_widget(GtkWidget *app) {
	CountData *cd;

	cd = g_new(CountData, 1);
	cd->displist = NULL;
	cd->displays = 1;
	cd->spacing = 10;
	cd->app = app;
	load_config(APPLET_WIDGET(app)->privcfgpath, cd);
	cd->orient = applet_widget_get_panel_orient (APPLET_WIDGET (cd->app));
	cd->globalbox = gtk_vbox_new(FALSE, 0);
	cd->counter = create_counter(cd);
	gtk_container_add(GTK_CONTAINER(cd->globalbox),cd->counter);
	applet_widget_add(APPLET_WIDGET(cd->app), cd->globalbox);
	gtk_widget_show_all(cd->app);

	gtk_signal_connect(GTK_OBJECT(cd->app), "delete_event",
		   	GTK_SIGNAL_FUNC(bye), cd);
	gtk_signal_connect(GTK_OBJECT(cd->app), "save_session",
		   	GTK_SIGNAL_FUNC(save_session), cd);
//	gtk_signal_connect(GTK_OBJECT(app),"change_orient",
//	           GTK_SIGNAL_FUNC(applet_change_orient), cd);

	applet_widget_register_stock_callback(APPLET_WIDGET(cd->app),
				      	"about",                    
				      	GNOME_STOCK_MENU_ABOUT,
				      	_("About"),         
				      	GTK_SIGNAL_FUNC(about_cb),
				      	cd);

	applet_widget_register_stock_callback(APPLET_WIDGET(cd->app),
				      	"preference",                    
				      	GNOME_STOCK_MENU_PREF,
				      	_("Preferences"),         
				      	GTK_SIGNAL_FUNC(prefs_cb),
				      	cd);
//	count_timer();
	return(cd);
}

static GtkWidget * applet_start_new_applet(const gchar *goad_id, const gchar **params, gint nparams)
{
	GtkWidget *app;
	CountData *cd;

fprintf(stderr, "applet_start_new_applet: %s\n", goad_id);
	if(strcmp(goad_id, "ledcount_applet")) return NULL;

	app = applet_widget_new(goad_id);
	if (!app)
		g_error("Can't create applet!\n");

	cd = create_count_widget(app);

	return app;
}

int
main(int argc, char *argv[])
{
	char *goad_id;
	GtkWidget *app;
	CountData *cd;

	/* initialize the i18n stuff */
	textdomain (PACKAGE);
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
	
	applet_widget_init("ledcount_applet", VERSION, argc, argv, NULL, 0, NULL);
	applet_factory_new("ledcount_applet_factory", NULL, applet_start_new_applet);
	goad_id = (char *)goad_server_activation_id();

fprintf(stderr, "goad_id = %s\n", goad_id);
	if(goad_id && !strcmp(goad_id, "ledcount_applet")) {
		app = applet_widget_new("ledcount_applet");
		if(!app)
			g_error("Cannot create ledcount applet!");

		cd = create_count_widget(app);
fprintf(stderr, "cre\n");
//		count_timer(NULL);
	}

	applet_widget_gtk_main();
	return(0);
}
