/*
 * vu-meter -- A volume units meter for esd
 * Copyright (C) 1998 Gregory McLean
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Cambridge, MA 
 * 02139, USA.
 *
 * Modified April 9 1999, by Dave J. Andruczyk to fix time sync problems
 * now it should be pretty much perfectly in sync with the audio.  Adjust
 * the "lag" variable below to taste if you don't like it...  Utilized
 * code from "extace" to get the desired effects..
 * 
 * Small additions April 14th 1999, by Dave J. Andruczyk to fix the missing 
 * peaks problem that happened with quick transients not showin on the vu-meter.
 * Now it will catch pretty much all of them.
 * 
 * Eye candy!
 */
#include <gnome.h>
#include <signal.h>
#include "gtkdisplay.h"

GtkWidget   *dial[2];
GtkWidget   *window;
gchar       *esd_host = NULL;

/* function prototypes to make gcc happy: */
void update (void);
void update_levels (gpointer data);
gint  update_display (gpointer data);
void  handle_read (gpointer data, gint source, GdkInputCondition condition);

void sig_alarm (int sig)
{
}

static void
error_close_cb(void)
{
    g_error("Cannot connect to sound daemon.");
}


int 
main (int argc, char *argv[])
{
    GnomeClient   *client;
    GtkWidget     *hbox;
    GtkWidget     *frame;
    gint          time_id;
    gint          i;
    gint          session_xpos = -1;
    gint          session_ypos = -1;
    gint          orient = 0;


	/* initialize the i18n stuff */
	textdomain (PACKAGE);
	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
fprintf(stderr, "%s\n", gettext("General"));

    gtk_init(&argc, &argv);
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title (GTK_WINDOW (window), _("Volume Meter"));
    if (session_xpos >=0 && session_ypos >= 0)
	gtk_widget_set_uposition (window, session_xpos, session_ypos);

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
			GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
    gtk_signal_connect (GTK_OBJECT (window), "delete_event",
			GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

    frame = gtk_frame_new (NULL);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_IN);
    gtk_container_border_width (GTK_CONTAINER (frame), 4);
    gtk_container_add (GTK_CONTAINER (window), frame);

    if ( !orient ) 
	hbox = gtk_vbox_new (FALSE, 5);
    else
	hbox = gtk_hbox_new (FALSE, 5);
    gtk_container_border_width (GTK_CONTAINER (hbox), 5);
    gtk_container_add (GTK_CONTAINER (frame), hbox);
    for (i = 0; i < 2; i++)
    {
	dial[i] = gtk_display_new ();
	gtk_display_set_value(dial[i], 8);
	gtk_widget_set_usize(dial[i], 100, 50);
	gtk_box_pack_start (GTK_BOX (hbox), dial[i], TRUE, TRUE, 0);
    }
    gtk_widget_show_all (window);

    gtk_main ();
    gtk_object_unref (GTK_OBJECT (client));
    gtk_timeout_remove (time_id);
    return 0;

}

